@@ -7,9 +7,10 @@ use std::{collections::HashMap, fmt::Debug, time::Instant};
 
 use crate::{
     component::{
+        bindings::{BindingQuery, DynamicBindings, KeySequenceSlice, NamedBindingQuery},
         layout::{LaidCanvas, LaidComponent, Layout},
         template::{ComponentId, DynamicMessage, DynamicProperties, Renderable},
-        BindingMatch, BindingTransition, LinkMessage, ShouldRender,
+        LinkMessage, ShouldRender,
     },
     terminal::{Canvas, Event, Key, Position, Rect, Size},
 };
@@ -277,41 +278,63 @@ impl App {
         let Self {
             ref mut components,
             ref subscriptions,
-            ref mut controller,
+            controller: ref mut input_controller,
             ..
         } = *self;
-        let mut clear_controller = false;
-        let mut changed_focus = false;
+        let mut clear_controller = true;
+        let mut binding_queries = SmallVec::<[_; 4]>::with_capacity(subscriptions.focused.len());
 
-        controller.push(key);
+        input_controller.push(key);
         for component_id in subscriptions.focused.iter() {
             let focused_component = components
                 .get_mut(component_id)
-                .expect("A focused component should be mounted.");
-            let binding = focused_component.input_binding(&controller.keys);
-            match binding.transition {
-                BindingTransition::Continue => {}
-                BindingTransition::Clear => {
-                    clear_controller = true;
-                }
-                BindingTransition::ChangedFocus => {
-                    changed_focus = true;
-                }
-            }
-            if let Some(message) = binding.message {
-                focused_component.update(message);
-            }
+                .expect("focused component to be mounted");
 
-            // If the focus has changed we don't notify other focused components
-            // deeper in the tree.
-            if changed_focus {
-                controller.keys.clear();
+            let binding_query = focused_component
+                .bindings
+                .keymap()
+                .check_sequence(&input_controller.keys);
+            binding_queries.push(binding_query.map(|binding_query| {
+                NamedBindingQuery::new(focused_component.bindings.keymap(), binding_query)
+            }));
+            match focused_component
+                .bindings
+                .keymap()
+                .check_sequence(&input_controller.keys)
+            {
+                Some(BindingQuery::Match(command_id)) => {
+                    if let Some(message) = focused_component.renderable.run_command(
+                        &focused_component.bindings,
+                        *command_id,
+                        &input_controller.keys,
+                    ) {
+                        focused_component.update(message);
+                    }
+                }
+                Some(BindingQuery::PrefixOf(prefix_of)) => {
+                    log::info!(
+                        "{} ({} commands)",
+                        KeySequenceSlice::from(input_controller.keys.as_slice()),
+                        prefix_of.len()
+                    );
+                    clear_controller = false;
+                }
+                None => {}
             }
         }
 
+        for component_id in subscriptions.notify.iter() {
+            let notify_component = components
+                .get_mut(component_id)
+                .expect("component to be mounted");
+            notify_component
+                .renderable
+                .notify_binding_queries(&binding_queries, &input_controller.keys);
+        }
+
         // If any component returned `BindingTransition::Clear`, we clear the controller.
         if clear_controller {
-            controller.keys.clear();
+            input_controller.keys.clear();
         }
     }
 
@@ -364,10 +387,12 @@ impl App {
                     let mut new_component = false;
                     let component = components.entry(component_id).or_insert_with(|| {
                         new_component = true;
-                        let renderable = template.create(component_id, frame, sender.clone_box());
+                        let (renderable, bindings) =
+                            template.create(component_id, frame, sender.clone_box());
                         MountedComponent {
                             renderable,
                             frame,
+                            bindings,
                             should_render: ShouldRender::Yes.into(),
                             generation,
                         }
@@ -388,10 +413,15 @@ impl App {
                         statistics.new += 1;
                     }
 
-                    if component.has_focus() {
+                    component.update_bindings();
+                    if component.bindings.focused() {
                         subscriptions.add_focused(component_id);
                     }
 
+                    if component.bindings.notify() {
+                        subscriptions.add_notify(component_id);
+                    }
+
                     if let Some(message) = component.tick() {
                         subscriptions.add_tickable(component_id, message);
                     }
@@ -428,6 +458,7 @@ impl App {
 
 struct ComponentSubscriptions {
     focused: SmallVec<[ComponentId; 2]>,
+    notify: SmallVec<[ComponentId; 2]>,
     tickable: SmallVec<[TickSubscription; 2]>,
 }
 
@@ -435,6 +466,7 @@ impl ComponentSubscriptions {
     fn new() -> Self {
         Self {
             focused: SmallVec::new(),
+            notify: SmallVec::new(),
             tickable: SmallVec::new(),
         }
     }
@@ -442,13 +474,21 @@ impl ComponentSubscriptions {
     #[inline]
     fn clear(&mut self) {
         self.focused.clear();
+        self.notify.clear();
         self.tickable.clear();
     }
 
+    #[inline]
     fn add_focused(&mut self, component_id: ComponentId) {
         self.focused.push(component_id);
     }
 
+    #[inline]
+    fn add_notify(&mut self, component_id: ComponentId) {
+        self.notify.push(component_id);
+    }
+
+    #[inline]
     fn add_tickable(&mut self, component_id: ComponentId, message: DynamicMessage) {
         self.tickable.push(TickSubscription {
             component_id,
@@ -491,6 +531,7 @@ type Generation = usize;
 struct MountedComponent {
     renderable: Box<dyn Renderable>,
     frame: Rect,
+    bindings: DynamicBindings,
     generation: Generation,
     should_render: bool,
 }
@@ -522,13 +563,8 @@ impl MountedComponent {
     }
 
     #[inline]
-    fn has_focus(&self) -> bool {
-        self.renderable.has_focus()
-    }
-
-    #[inline]
-    fn input_binding(&self, pressed: &[Key]) -> BindingMatch<DynamicMessage> {
-        self.renderable.input_binding(pressed)
+    fn update_bindings(&mut self) {
+        self.renderable.bindings(&mut self.bindings)
     }
 
     #[inline]
